#include <linux/init.h>
#include <linux/smp.h>
#include <linux/spinlock.h>
#include <linux/kallsyms.h>
#include <linux/bootmem.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/proc_fs.h>
#include <linux/mm.h>
#include <linux/mm_types.h>
#include <linux/sched.h>
#include <linux/mmzone.h>
#include <linux/pagemap.h>
#include <linux/seq_file.h>
/*
#include <linux/config.h>
*/
#include <linux/kprobes.h>
#include <linux/smp_lock.h>

#include <asm/traps.h>
#include <asm/ptrace.h>
#include <asm/stacktrace.h>
#include <cameo/define.h>
#include <cameo/color_dump.h>
#include <cameo/env_dump.h>
#include <cameo/memmap_pid.h>
#include <cameo/memrw.h>

/*CAMEOTAG:Add by Juncheng, 2014-5-4*/
#define CDD_PROC_FILE	            "cdd"
#define CDD_CRASH_FILE_NAME_SIZE    256
#define CDD_CRASH_BUF_LEN           1024


#define MEMMAP_PROC_FILE	"memmap_pid"

typedef struct{
    int     status;
    int     pid;
    char    name[CDD_CRASH_FILE_NAME_SIZE];   
}tCddInfo;

#define CDD_MEMSET memset  
#define CDD_SPRINTF sprintf
#define CDD_VFS_WRITE \
    set_fs(KERNEL_DS);vfs_write 
	
/*CAMEOTAG:Add by Juncheng, 2014-5-4*/
static ssize_t cdd_read(struct file *file, char __user *data, size_t buflen, loff_t *fpos);
static ssize_t cdd_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos);
static int cdd_open(struct inode *inode, struct file *filp);

/*CAMEOTAG:Add by Juncheng, 2014-5-4*/
static tCddInfo                    cddInfo = {0};
static struct file                 *pCddFp = NULL;
static char                        cddBuf[CDD_CRASH_BUF_LEN] = {0};
static loff_t                      cddPos = 0; 
static mm_segment_t                old_fs = {0};
static int                         isFileOpen = 0;
static int                         isPrintSymbolWanted = 0;


extern const struct file_operations proc_memmap_pid_operations;
void cdd_print_symbol(const char *fmt, char *buffer)
{
    if(cddInfo.status != 1)
        return;
    if(isPrintSymbolWanted != 1)
        return;
    if(isFileOpen != 1)
    {
                pCddFp = filp_open(cddInfo.name,
                            O_RDWR|O_CREAT, 0644);
                if(IS_ERR(pCddFp))
                {
                    printk("\n[OS]CDD create file error\n");
                    return;
                }
                isFileOpen = 1;
    }
    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, fmt, buffer);   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);
}

void cdd_crash_regs(struct pt_regs *regs)
{
	const int field = 2 * sizeof(unsigned long);
	unsigned int cause = regs->cp0_cause;
	int i;
    
    /*Send signal to cdd for saving message/packet*/
    kill_proc(cddInfo.pid, SIGUSR2, 1);
    
    if(cddInfo.status != 1)
        return;

    if(isFileOpen != 1)
    {
                pCddFp = filp_open(cddInfo.name,
                            O_RDWR|O_CREAT, 0644);
                if(IS_ERR(pCddFp))
                {
                    printk("\n[OS]CDD create file error\n");
                    return;
                }
                isFileOpen = 1;
    }
    /*To print symbol to file*/
    isPrintSymbolWanted = 1;
    old_fs = get_fs();
    
    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "Cpu %d\n", smp_processor_id());   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);

	/*
	 * Saved main processor registers
	 */
	for (i = 0; i < 32; ) {
		if ((i % 4) == 0)
		{
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "$%2d   :", i);   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (i == 0)
		{
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, " %0*lx", field, 0UL);   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		else if (i == 26 || i == 27)
		{
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, " %*s", field, "");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		else
		{
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, " %0*lx", field, regs->regs[i]);   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}

		i++;
		if ((i % 4) == 0)
		{
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "\n");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
	}

    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "Hi    : %0*lx\n", field, regs->hi);   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);

    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "Lo    : %0*lx\n", field, regs->lo);   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);

	/*
	 * Saved cp0 registers
	 */
    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "epc   : %0*lx ", field, regs->cp0_epc);   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);
	print_symbol("%s ", regs->cp0_epc);
    
    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "    %s\n", print_tainted());   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);

    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "ra    : %0*lx ", field, regs->regs[31]);   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);
	print_symbol("%s\n", regs->regs[31]);

    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "Status: %08x    ", (uint32_t) regs->cp0_status);   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);

	if (current_cpu_data.isa_level == MIPS_CPU_ISA_I) {
		if (regs->cp0_status & ST0_KUO)
		{
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "KUo ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (regs->cp0_status & ST0_IEO)
		{
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "IEo ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (regs->cp0_status & ST0_KUP)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "KUp ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (regs->cp0_status & ST0_IEP)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "IEp ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (regs->cp0_status & ST0_KUC)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "KUc ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (regs->cp0_status & ST0_IEC)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "IEc ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
	} else {
		if (regs->cp0_status & ST0_KX)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "KX ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (regs->cp0_status & ST0_SX)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "SX ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (regs->cp0_status & ST0_UX)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "UX ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		switch (regs->cp0_status & ST0_KSU) {
		case KSU_USER:
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "USER ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
			break;
		case KSU_SUPERVISOR:
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "SUPERVISOR ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
			break;
		case KSU_KERNEL:
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "KERNEL ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
			break;
		default:
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "BAD_MODE ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
			break;
		}
		if (regs->cp0_status & ST0_ERL)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "ERL ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (regs->cp0_status & ST0_EXL)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "EXL ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
		if (regs->cp0_status & ST0_IE)
			
        {
            CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
            CDD_SPRINTF(cddBuf, "IE ");   
	        CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
            set_fs(old_fs);
		}
	}
    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "\n");   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);

    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "Cause : %08x\n", cause);   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);

	cause = (cause & CAUSEF_EXCCODE) >> CAUSEB_EXCCODE;
	if (1 <= cause && cause <= 5)
	{
        CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
        CDD_SPRINTF(cddBuf, "BadVA : %0*lx\n", field, regs->cp0_badvaddr);   
	    CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
        set_fs(old_fs);
	}

    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf, "PrId  : %08x\n", read_c0_prid());   
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);

    if(isFileOpen == 1)
    {
        filp_close(pCddFp, NULL);
        isFileOpen = 0;
    }
    /*To print symbol to file*/
    isPrintSymbolWanted = 0;
}

void cdd_crash_trace_callback(struct task_struct *task, struct pt_regs *regs)
{
	struct pt_regs *reg_thread = task_pt_regs(task);
	unsigned long epc_val = 0, epc_saved, ra_val, dummy;
	int ret = 0, ra_pos = 0, sp_size = 0;
	unsigned long sp = reg_thread->regs[29];

	epc_val = reg_thread->cp0_epc;
	dummy = ra_val = reg_thread->regs[31];	/* RA init for leave function */

    if(cddInfo.status != 1)
        return;
    
    if(isFileOpen != 1)
    {
                pCddFp = filp_open(cddInfo.name,
                            O_RDWR|O_CREAT, 0644);
                if(IS_ERR(pCddFp))
                {
                    printk("\n[OS]CDD create file error\n");
                    return;
                }
                isFileOpen = 1;
    }
    
	do {
		/* first, get RA, then sp_size */
		/*   also get RA-Position when get RA,
		 *        get stack size at second read */
		epc_saved = epc_val;
		ret = get_valsp(epc_val, &ra_pos, &ra_val, sp, task, 0xafbf0000);//, (unsigned char*)regs->regs[29]
		if (ret <= 0) {
			break;
		}
		epc_val = ret;
		ret = get_valsp(epc_val, &sp_size, &dummy, sp, task, 0x27bd0000);
		if (ret <= 0) {
			break;
		}
		if (sp_size < 0) {	/* abs */
			sp_size = -sp_size;
		}
        CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
        CDD_SPRINTF(cddBuf,"%08lx (size: %ld, sp: %08lx, ra: %08lx)\n", epc_saved, (long int)sp_size, sp, ra_val);
	    CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
        set_fs(old_fs);
		epc_val = ra_val;	/* when leave function, first call get_valsp, will keep use init RA */
		sp += sp_size;
		if (epc_saved == epc_val) {
			break;
		}
	} while (ret > 0);
    
    CDD_MEMSET(cddBuf, 0, sizeof(cddBuf));
    CDD_SPRINTF(cddBuf,"all backtrace dump fine, only FYI\n");    
	CDD_VFS_WRITE(pCddFp,cddBuf, strlen(cddBuf), &cddPos);
    set_fs(old_fs);

    if(isFileOpen == 1)
    {
        filp_close(pCddFp, NULL);
        isFileOpen = 0;
    }
}


/*CAMEOTAG:Add by Juncheng, 2014-5-4*/
static ssize_t cdd_read(struct file *file, char __user *data, size_t buflen, loff_t *fpos)
{
	return 0;
}

static ssize_t cdd_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
    memset(&cddInfo, 0, sizeof(cddInfo));
    if(copy_from_user(&cddInfo, buf, count))
    {
        goto fail_copy;
    }
fail_copy:
    return count;
}

static int cdd_open(struct inode *inode, struct file *filp)
{
    return 0;
}

/*CAMEOTAG:Add by Juncheng, 2014-5-4*/
const struct file_operations proc_cdd_operations = {
	.read		= cdd_read,
	.open		= cdd_open,
	.write		= cdd_write,
};

static int __init cdd_crash_init(void)
{   
	/*CAMEOTAG:Add by Juncheng, 2014-5-4*/
    struct proc_dir_entry *proc_cdd_hdl = NULL;
	struct proc_dir_entry *proc_memmap_pid_hdl = NULL;

	seep("cdd_crash load\n");

    /*CAMEOTAG:Add by Juncheng, 2014-5-4*/
    proc_cdd_hdl = create_proc_entry(CDD_PROC_FILE, S_IWGRP|S_IWUSR|S_IWOTH, NULL);
	if (proc_cdd_hdl) {
		proc_cdd_hdl->proc_fops = &proc_cdd_operations;
	} else {
		seep("cdd register proc file fail\n");
	}


	
	seep("memmap_pid load\n");
	/* maybe fast, if use seq_file */
	proc_memmap_pid_hdl = create_proc_entry(MEMMAP_PROC_FILE, S_IRUSR, NULL);
	if (proc_memmap_pid_hdl) {
		proc_memmap_pid_hdl->proc_fops = &proc_memmap_pid_operations;
	} else {
		seep("memmap_id register proc file fail\n");
	}

	/*Bind the hooks*/
	MemmapPidDumpBind(memmap_pid_dump);
	CDDPrintSymbolBind(cdd_print_symbol);

	seep("Init CAMEO Debug Trace Driver Module....OK\n");
	return 0;
}

static void __exit cdd_crash_exit(void)
{
	seep("cdd_crash unload\n");
    remove_proc_entry(CDD_PROC_FILE, NULL);

	seep("memmap_pid unload\n");
	remove_proc_entry(MEMMAP_PROC_FILE, NULL);
	return;
}
module_init(cdd_crash_init);
module_exit(cdd_crash_exit);
MODULE_LICENSE("GPL");

